@@ -1,14 +1,20 @@
 import os
 import time
 from pathlib import Path
-from typing import Dict, Iterator, Optional, Union
+from typing import Dict, Iterator, List, Optional, Union
 
 import requests
 
 from darwin.config import Config
 from darwin.dataset import RemoteDataset
 from darwin.dataset.identifier import DatasetIdentifier
-from darwin.exceptions import InsufficientStorage, InvalidLogin, MissingConfig, NotFound, Unauthorized
+from darwin.exceptions import (
+    InsufficientStorage,
+    InvalidLogin,
+    MissingConfig,
+    NotFound,
+    Unauthorized,
+)
 from darwin.utils import is_project_dir, urljoin
 from darwin.validators import name_taken, validation_error
 
@@ -19,7 +25,7 @@ class Client:
         self.url = config.get("global/api_endpoint")
         self.base_url = config.get("global/base_url")
         self.default_team = default_team or config.get("global/default_team")
-        self.features = {}
+        self.features: dict = {}
 
     def get(
         self, endpoint: str, team: Optional[str] = None, retry: bool = False, raw: bool = False, debug: bool = False
@@ -49,6 +55,7 @@ class Client:
         Unauthorized
             Action is not authorized
         """
+
         response = requests.get(urljoin(self.url, endpoint), headers=self._get_headers(team))
 
         if response.status_code == 401:
@@ -70,7 +77,15 @@ class Client:
         else:
             return self._decode_response(response, debug)
 
-    def put(self, endpoint: str, payload: Dict, team: Optional[str] = None, retry: bool = False, debug: bool = False):
+    def put(
+        self,
+        endpoint: str,
+        payload: Dict,
+        team: Optional[str] = None,
+        retry: bool = False,
+        debug: bool = False,
+        raw: bool = False,
+    ):
         """Put something on the server trough HTTP
 
         Parameters
@@ -83,6 +98,8 @@ class Client:
             Retry to perform the operation. Set to False on recursive calls.
         debug : bool
             Debugging flag. In this case failed requests get printed
+        raw : bool
+            Flag for returning raw response
 
         Returns
         -------
@@ -110,7 +127,10 @@ class Client:
             time.sleep(10)
             return self.put(endpoint, payload=payload, retry=False)
 
-        return self._decode_response(response, debug)
+        if raw:
+            return response
+        else:
+            return self._decode_response(response, debug)
 
     def post(
         self,
@@ -207,7 +227,7 @@ class Client:
                     f"Client get request response ({response.json()}) with unexpected status "
                     f"({response.status_code}). "
                     f"Client: ({self})"
-                    f"Request: (endpoint={endpoint}, payload={payload})"
+                    f"Request: (endpoint={endpoint})"
                 )
             if retry:
                 time.sleep(10)
@@ -267,8 +287,8 @@ class Client:
         RemoteDataset
             Initialized dataset
         """
-        if isinstance(dataset_identifier, str):
-            dataset_identifier = DatasetIdentifier.parse(dataset_identifier)
+        dataset_identifier = DatasetIdentifier.parse(dataset_identifier)
+
         if not dataset_identifier.team_slug:
             dataset_identifier.team_slug = self.default_team
 
@@ -330,7 +350,7 @@ class Client:
         self.features[team_slug] = self.get(f"/teams/{team_slug}/features")
 
     def feature_enabled(self, feature_name: str, team: Optional[str] = None):
-        team_slug = self.config.get_team(team or self.default_team)["slug"]
+        team_slug: str = self.config.get_team(team or self.default_team)["slug"]
         if team_slug not in self.features:
             self.load_feature_flags(team)
         for feature in self.features[team_slug]:
