@@ -1,18 +1,17 @@
 import itertools
 import json
 import multiprocessing as mp
-import sys
 from collections import Counter, defaultdict
 from pathlib import Path
-from typing import Dict, Generator, Iterable, List, Optional, Set, Union
+from typing import Dict, Generator, List, Optional, Set, Tuple, Union
 
 import numpy as np
-from PIL import Image
-from tqdm import tqdm
-
 from darwin.exceptions import NotFound
 from darwin.importer.formats.darwin import parse_file
 from darwin.utils import SUPPORTED_EXTENSIONS, SUPPORTED_VIDEO_EXTENSIONS
+from PIL import Image
+from rich.live import Live
+from rich.progress import ProgressBar, track
 
 
 def get_release_path(dataset_path: Path, release_name: Optional[str] = None):
@@ -46,7 +45,7 @@ def get_release_path(dataset_path: Path, release_name: Optional[str] = None):
     return release_path
 
 
-def extract_classes(annotations_path: Path, annotation_type: str):
+def extract_classes(annotations_path: Path, annotation_type: str) -> Tuple[Dict[str, Set[int]], Dict[int, Set[str]]]:
     """
     Given a the GT as json files extracts all classes and an maps images index to classes
 
@@ -62,25 +61,29 @@ def extract_classes(annotations_path: Path, annotation_type: str):
     classes: dict
     Dictionary where keys are the classes found in the GT and values
     are a list of file numbers which contain it
-    idx_to_classes: dict
+    indices_to_classes: dict
     Dictionary where keys are image indices and values are all classes
     contained in that image
     """
-    assert annotation_type in ["tag", "polygon", "bounding_box"]
 
-    classes = defaultdict(set)
-    indices_to_classes = defaultdict(set)
-    annotation_files = list(annotations_path.glob("**/*.json"))
-    for i, file_name in enumerate(annotation_files):
-        with open(file_name) as f:
-            annotations = json.load(f)["annotations"]
-            if annotations:
-                for annotation in annotations:
-                    if annotation_type not in annotation:
-                        continue
-                    class_name = annotation["name"]
-                    indices_to_classes[i].add(class_name)
-                    classes[class_name].add(i)
+    assert annotation_type in ["bounding_box", "polygon", "tag"]
+
+    classes: Dict[str, Set[int]] = defaultdict(set)
+    indices_to_classes: Dict[int, Set[str]] = defaultdict(set)
+
+    for i, file_name in enumerate(sorted(annotations_path.glob("**/*.json"))):
+        annotation_file = parse_file(file_name)
+        if not annotation_file:
+            continue
+
+        for annotation in annotation_file.annotations:
+            if annotation.annotation_class.annotation_type != annotation_type:
+                continue
+
+            class_name = annotation.annotation_class.name
+            indices_to_classes[i].add(class_name)
+            classes[class_name].add(i)
+
     return classes, indices_to_classes
 
 
@@ -173,19 +176,20 @@ def exhaust_generator(progress: Generator, count: int, multi_threaded: bool):
     """
     responses = []
     if multi_threaded:
-        pbar = tqdm(total=count)
+        pbar = ProgressBar(total=count)
 
         def update(*a):
-            pbar.update()
+            pbar.completed += 1
 
-        with mp.Pool(mp.cpu_count()) as pool:
-            for f in progress:
-                responses.append(pool.apply_async(_f, args=(f,), callback=update))
-            pool.close()
-            pool.join()
-        responses = [response.get() for response in responses if response.successful()]
+        with Live(pbar):
+            with mp.Pool(mp.cpu_count()) as pool:
+                for f in progress:
+                    responses.append(pool.apply_async(_f, args=(f,), callback=update))
+                pool.close()
+                pool.join()
+            responses = [response.get() for response in responses if response.successful()]
     else:
-        for f in tqdm(progress, total=count, desc="Progress"):
+        for f in track(progress, total=count, description="Progress"):
             responses.append(_f(f))
     return responses
 
@@ -256,6 +260,7 @@ def get_annotations(
     annotation_type: str = "polygon",
     release_name: Optional[str] = None,
     annotation_format: Optional[str] = "coco",
+    ignore_inconsistent_examples: bool = False,
 ):
     """
     Returns all the annotations of a given dataset and split in a single dictionary
@@ -276,6 +281,11 @@ def get_annotations(
         Version of the dataset
     annotation_format: str
         Re-formatting of the annotation when loaded [coco, darwin]
+    ignore_inconsistent_examples: bool
+        Ignore examples for which we have annotations, but either images are missing,
+        or more than one images exist for the same annotation.
+        If set to `True`, then filter those examples out of the dataset.
+        If set to `False`, then raise an error as soon as such an example is found.
 
     Returns
     -------
@@ -326,6 +336,7 @@ def get_annotations(
     annotations_paths = []
 
     # Find all the annotations and their corresponding images
+    invalid_annotation_paths = []
     for stem in stems:
         annotation_path = annotations_dir / f"{stem}.json"
         images = []
@@ -333,17 +344,27 @@ def get_annotations(
             image_path = images_dir / f"{stem}{ext}"
             if image_path.exists():
                 images.append(image_path)
+                continue
             image_path = images_dir / f"{stem}{ext.upper()}"
             if image_path.exists():
                 images.append(image_path)
-        if len(images) < 1:
+
+        image_count = len(images)
+        if image_count != 1 and ignore_inconsistent_examples:
+            invalid_annotation_paths.append(annotation_path)
+            continue
+        elif image_count < 1:
             raise ValueError(f"Annotation ({annotation_path}) does not have a corresponding image")
-        if len(images) > 1:
+        elif image_count > 1:
             raise ValueError(f"Image ({stem}) is present with multiple extensions. This is forbidden.")
 
         images_paths.append(images[0])
         annotations_paths.append(annotation_path)
 
+    print(f"Found {len(invalid_annotation_paths)} invalid annotations")
+    for p in invalid_annotation_paths:
+        print(p)
+
     if len(images_paths) == 0:
         raise ValueError(f"Could not find any {SUPPORTED_EXTENSIONS} file" f" in {dataset_path / 'images'}")
 
@@ -487,3 +508,15 @@ def compute_distributions(
                 instance_distribution[partition] += Counter(annotation_class_names)
 
     return {"class": class_distribution, "instance": instance_distribution}
+
+
+# https://github.com/python/cpython/blob/main/Lib/pathlib.py#L812
+# TODO implemented here because it's not supported in Pythton < 3.9
+def is_relative_to(path: Path, *other):
+    """Return True if the path is relative to another path or False.
+    """
+    try:
+        path.relative_to(*other)
+        return True
+    except ValueError:
+        return False
