@@ -1,10 +1,9 @@
 from pathlib import Path
 from typing import Callable, List, Union
 
-from tqdm import tqdm
-
 import darwin.datatypes as dt
 from darwin.utils import secure_continue_request
+from rich.progress import track
 
 
 def build_main_annotations_lookup_table(annotation_classes):
@@ -92,7 +91,7 @@ def import_annotations(
     if local_files_missing_remotely:
         print(f"{len(local_files_missing_remotely)} file(s) are missing from the dataset")
         for local_file in local_files_missing_remotely:
-            print(f"\t{local_file.path}: '{local_file.filename}'")
+            print(f"\t{local_file.path}: '{local_file.full_path}'")
 
         if not secure_continue_request():
             return
@@ -132,8 +131,9 @@ def import_annotations(
         if type(parsed_files) is not list:
             parsed_files = [parsed_files]
         # remove files missing on the server
-        parsed_files = [parsed_file for parsed_file in parsed_files if parsed_file not in local_files_missing_remotely]
-        for parsed_file in tqdm(parsed_files):
+        missing_files = [missing_file.full_path for missing_file in local_files_missing_remotely]
+        parsed_files = [parsed_file for parsed_file in parsed_files if parsed_file.full_path not in missing_files]
+        for parsed_file in track(parsed_files):
             image_id = remote_files[parsed_file.full_path]
             _import_annotations(
                 dataset.client, image_id, remote_classes, attributes, parsed_file.annotations, dataset, append
